#include "XYZDistributionTracksProcessor.h"

//C++
#include <iostream>
using namespace std;

//LCIO
#include <IMPL/LCCollectionVec.h>
#include <EVENT/Track.h>
#include <EVENT/TrackerHit.h>
#include "IntersectionCalculator.h"

// Aida
//#ifdef MARLIN_USE_AIDA // the whole processor does not make sense without aida
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
// endif

#include <marlin/Global.h>

#include "TrackFitterFactory.h"
#include "TrackFitterBase.h"

using namespace lcio ;
using namespace marlin ;

namespace marlintpc
{

XYZDistributionTracksProcessor aXYZDistributionTracksProcessor;
    
XYZDistributionTracksProcessor::XYZDistributionTracksProcessor(): Processor("XYZDistributionTracksProcessor")
						    , _distributionXHisto(0) , _distributionYHisto(0), _distributionZHisto(0), _distributionXYHisto(0)
{
    // register steering parameters: name, description, class-variable, default value
    registerInputCollection(    LCIO::TRACK,
				"InputTracks" , 
				"Name of the input tracks collection"  ,
				_inputColName ,
				std::string("TPCTracks") ) ;
    // modify processor description
    _description = "XYZDistributionTracksProcessor: Fill x, y, and z distribution of hits in tracks into AIDA histgram" ;

  registerProcessorParameter( "HistogramMinValueX" , 
			      "Minimum value in x histogram (default: 0. mm)"  ,
			      _histogramMinValueX ,
			      float(0.) ) ;

  registerProcessorParameter( "HistogramMaxValueX" , 
			      "Maximal value in x histogram (default:  15. mm)"  ,
			      _histogramMaxValueX ,
			      float(15.) ) ;
  registerProcessorParameter( "HistogramMinValueY" , 
			      "Minimum value in y histogram (default: 0. mm)"  ,
			      _histogramMinValueY ,
			      float(0.) ) ;

  registerProcessorParameter( "HistogramMaxValueY" , 
			      "Maximal value in y histogram (default:  15. mm)"  ,
			      _histogramMaxValueY ,
			      float(15.) ) ;

  registerProcessorParameter( "HistogramMinValueZ" , 
			      "Minimum value in z histogram (default: 0.0 mm)"  ,
			      _histogramMinValueZ ,
			      float(0.0) ) ;

  registerProcessorParameter( "HistogramMaxValueZ" , 
			      "Maximal value in z histogram (default:  260.0 mm)"  ,
			      _histogramMaxValueZ ,
			      float(260.0) ) ;
}

XYZDistributionTracksProcessor::~XYZDistributionTracksProcessor()
{
}

void XYZDistributionTracksProcessor::init()
{ 
	// usually a good idea to
	printParameters() ;

	// get a histograms from the AIDAProcessor
	_distributionXHisto =   AIDAProcessor::histogramFactory(this)->
	  createHistogram1D( "xDistributionTracksHisto", 
			     "xDistributionTracksHisto", 
			     100, _histogramMinValueX, _histogramMaxValueX);

	_distributionYHisto =   AIDAProcessor::histogramFactory(this)->
	  createHistogram1D( "yDistributionTracksHisto", 
			     "yDistributionTracksHisto", 
			     100, _histogramMinValueY, _histogramMaxValueY);

	_distributionZHisto =   AIDAProcessor::histogramFactory(this)->
	  createHistogram1D( "zDistributionTracksHisto", 
			     "zDistributionTracksHisto", 
			     100, _histogramMinValueZ, _histogramMaxValueZ);
	_distributionXYHisto =   AIDAProcessor::histogramFactory(this)->
	  createHistogram2D( "xyDistributionTracksHisto", 
			     "xyDistributionTracksHisto", 
			     900, _histogramMinValueX, _histogramMaxValueX,
			     900, _histogramMinValueY, _histogramMaxValueY);
    }

void XYZDistributionTracksProcessor::processRunHeader( EVENT::LCRunHeader* run )
{
}
    
void XYZDistributionTracksProcessor::processEvent( LCEvent * evt ) 
{ 
    //read input data

    // we need a LCCollectionVec to use iterators, as LCCollectio is not a std::vector
    LCCollectionVec* InputCollection=0;
    try{
	InputCollection = dynamic_cast<LCCollectionVec*>(evt->getCollection( _inputColName )) ;
    }
    catch(DataNotAvailableException &e)
    {
	// no input data
	// just continue ...
	return;
    }


    // cast did not succeed
    if( InputCollection == 0 )
    {
	streamlog_out(ERROR2) << "InputCollection vector is 0" << std::endl;
	return;
    }

    for(LCCollectionVec::iterator it = InputCollection->begin() ; 
	it < InputCollection->end(); it ++ )
    {
	
	Track * track = dynamic_cast<Track *>(*it) ;
	
	TrackerHitVec hitVector = track->getTrackerHits();
	
//	    cout << "nr of hits " << hitVector.size()<< endl;
	
//	    int hitNr=0;
	
//	    cout << "\t\t hit\t x \t y \t z \t residual_x/r*phi" << endl;
	for (TrackerHitVec::iterator hit = hitVector.begin();
	     hit <  hitVector.end(); hit ++)
	{
	    const double *pos = (*hit)->getPosition();
	    
	    _distributionXHisto->fill(pos[0]);
	    _distributionYHisto->fill(pos[1]);
	    _distributionZHisto->fill(pos[2]);
//	    if (pos[2] < 10)
		_distributionXYHisto->fill(pos[0],pos[1]);

	}
    }
    
}


}// namespace 
