/*
 * PathFinderInterface.h
 *
 *  Created on: Nov 4, 2011
 *      Author: caiazza
 */

#ifndef PATHFINDERINTERFACEPROCESSOR_H_
#define PATHFINDERINTERFACEPROCESSOR_H_

#include <marlin/Processor.h>
#include <lcio.h>
#include <string>

//Pathfinder includes
#include "FinderParameter.h"
#include "HoughTrafoTrackFinder.h"
#include "basicHit.h"

namespace marlintpc
{
class PathFinderInterfaceProcessor : public marlin::Processor
{
public:
    virtual Processor*  newProcessor() { return new PathFinderInterfaceProcessor; }

    PathFinderInterfaceProcessor();

    virtual ~PathFinderInterfaceProcessor(){};

    virtual void init();

    virtual void processRunHeader(EVENT::LCRunHeader* run);

    virtual void processEvent(EVENT::LCEvent* evt);

    virtual void check(EVENT::LCEvent* evt);

    virtual void end();

protected:
    /**
     * Input Collection Name
     */
    std::string _inputColName;

    /**
     * Output collection name
     */
    std::string _outColName;

    /**
     *  Set the persistency flag of the output collection.
     */
    bool _outputIsPersistent;

    /**
     * Defines the type of track to search. The following values are valid:
     * 0: Straight Track 1:Helix Track (Default) 2:Curlers
     */
    int _trackModel;

    /**
     * Defines the minimum number of hits of a selected track.
     */
    int _minHitonTrack;

    /**
     * Defines the maximum allowed distance of any hit from the candidate track
     * in the xy projection in mm
     */
    float _maxHitDistXY;

    /**
     * Defines the maximum allowed distance of any hit from the candidate track
     * in the z direction in mm
     */
    float _maxHitDistZ;

    /**
     * Select the level of refinement of the binning of the Hough Map used for the track finding algorithm
     * This value can be set between 1 and 10 and the processor will calculate the actual number used
     * for each of the binning variables internally.
     * The value must range between 1 and 10 where 1 is the coarsest resolution and 10 the finest
     */
    int _binningLevel;

    /**
     * Select whether to activate or not the neighborhood search feature of the PATHFINDER package
     */
    bool _activateNeighSearch;

    /**
     * Select whether to calculate a temporary origin in the center of mass of the hits to increase the precision of the algorithm
     * NOT YET IMPLEMENTED
     */
    bool _activateCoordRedef;

    /*
     * Expert mode: The user can override the binning value set in the simplified mode.
     * None of these value must exceed 1000
     */
    int _overrideD0XYBins;
    int _overrideD0SZBins;
    int _overrideThetaXYBins;
    int _overrideThetaSZBins;
    int _overrideOmegaBins;

protected:
    FinderParameter* _myFinderParameter;
    HoughTrafoTrackFinder* _trackFinder;

private:
    int _nRun;
    int _nEvt;
};

}//end class PathFinderInterfaceProcessor
#endif /* PATHFINDERINTERFACE_H_ */
