#ifndef TRACKCONNECTION_H
#define TRACKCONNECTION_H 1

#include "lcio.h"
#include <string>
#include <map>

#include <EVENT/TrackerHit.h>
#include <IMPL/TrackerHitImpl.h>

namespace marlintpc
{

using namespace lcrtrel;

class TrackConnection
{

    public:

        /** Default constructor
        */
        TrackConnection();

        /** Destructor
        */
        ~TrackConnection();

        /** UsedFlag
        */
        struct UsedFlag : LCIntExtension<UsedFlag> {} ;

        /** Track merger process
         */
        void mergeTracks(IMPL::TrackImpl*& mergedTrack, std::multimap<int, IMPL::TrackImpl*> &tracksByModuleID);

        void setConeAngle(double angle);

    private:

        /** Merge the TrackerHits
         */
        std::multimap<int, IMPL::TrackImpl*> *_tracksByModuleID;

        /** Check Track connection
         */
        bool connectSegment(IMPL::TrackImpl* mergedTrack, std::multimap<int, IMPL::TrackImpl*>::iterator nextSegment);

        /** Merge decision
         */
        void connect(IMPL::TrackImpl* mergedTrack, std::multimap<int, IMPL::TrackImpl*>::iterator nextSegment);

        /** Sorting algorithm
         */
#if 0   // 2D
        class Compare
        {
            public:
                bool operator()(EVENT::TrackerHit* left,  EVENT::TrackerHit* right)
                {
                    const double* xv1 = left->getPosition();
                    const double* xv2 = right->getPosition();
                    double r12 = xv1[0] * xv1[0] + xv1[1] * xv1[1];
                    double r22 = xv2[0] * xv2[0] + xv2[1] * xv2[1];
                    return r12 < r22;
                }
        };

#else   // 3D
        class Compare
        {
            public:
                bool operator()(EVENT::TrackerHit* left,  EVENT::TrackerHit* right)
                {
                    const double* xv1 = left->getPosition();
                    const double* xv2 = right->getPosition();
                    double r12 = xv1[0] * xv1[0] + xv1[1] * xv1[1] + xv1[2] * xv1[2];
                    double r22 = xv2[0] * xv2[0] + xv2[1] * xv2[1] + xv2[2] * xv2[2];
                    return r12 < r22;
                }
        };
#endif

        /** For Connection Condition
         */
        const double* _x1;
        const double* _x3;

        double _coneAngle;

};

}// namespace marlintpc

#endif // TRACKCONNECTION_H

