#ifdef USE_TRACK_FITTER_KALMAN
#include "TPCHitDomain.h"
#include <gear/TPCParameters.h>
#include <marlin/Global.h>
#include <gear/TPCModule.h>
#include <streamlog/streamlog.h>

using namespace marlin;
namespace marlintpc
{
TPCHitDomain::TPCHitDomain(const TPCParameters* tp, int maxSkip)
        : _tpcParameters(tp), _MAX_SKIP(maxSkip)
{
    _skip = _MAX_SKIP;
}


bool TPCHitDomain::setDomain(const ModuleRowHit& hit, int step)
{
    streamlog_out(DEBUG3) << "In SetDomain()" << endl;

    _mod = (hit.first).first;
    _row = (hit.first).second + step;

    const TPCParameters* tpcParameters = 0;
    if (!tpcParameters)tpcParameters = &Global::GEAR->getTPCParameters();
    const TPCModule* tpcModule = & tpcParameters->getModule(_mod);
    int nRows = tpcModule->getNRows();

    if (_row >= nRows)return atBoundary(hit);

    streamlog_out(DEBUG3) << "mod=" << _mod << "  row=" << _row << endl;
    return true;
}


bool TPCHitDomain::atBoundary(const ModuleRowHit& hit)
{
    /*
      This is for LP1 beamtest
      1.get cartesian hit position, and add a step to it
      2.get the nearest module for the position
      3.get the first layer of the module
    */

    const double* pos = TPCHitMap::getHitPosition(hit);
    double newHitPos[3];

    ///////////////FIXME//////////////
    const double BOUNDARY_GAP = 60.;
    newHitPos[0] = pos[0] + BOUNDARY_GAP;
    newHitPos[1] = pos[1];
    newHitPos[2] = pos[2];
    ///////////////FIXME//////////////

    streamlog_out(DEBUG3) << "Expected position in the next module=" << newHitPos[0] << ", " << newHitPos[1] << endl;
    if (_tpcParameters->isInsideModule(newHitPos[0], newHitPos[1]))
        {
            const TPCModule& tpcMod = _tpcParameters->getNearestModule(newHitPos[0], newHitPos[1]);

            _mod = tpcMod.getModuleID();
            _row = 0;
            return true;
        }
    else
        {
            streamlog_out(DEBUG3) << "Expected position is not inside any module!" << endl;
            return false;
        }
}


bool TPCHitDomain::skipDomain(const ModuleRowHit& hit)
{
    streamlog_out(DEBUG3) << "In SkipDomain()" << endl;

    if (_skip > 0)
        {
            _row++;
            _skip--;
            streamlog_out(DEBUG3) << "skip a row..." << endl;

            const TPCParameters* tpcParameters = 0;
            if (!tpcParameters)tpcParameters = &Global::GEAR->getTPCParameters();
            const TPCModule* tpcModule = & tpcParameters->getModule(_mod);
            int nRows = tpcModule->getNRows();

            if (_row >= nRows)
                return atBoundary(hit);
            else
                return true;
        }
    else
        return false;
}


}
#endif
