
// MarlinTPC
#include "ChargeSignal.h"

using namespace std;

namespace marlintpc {

/**
* constructor
*/
ChargeSignal::ChargeSignal(int charge_id, int pad_id, int module_id):
	_charge_id( charge_id ),
	_pad_id( pad_id ),
	_module_id( module_id ),
	_electron_count( 0 ),
	_mean_time( 0. ),
	_S( 0. ) {
}


/** adds an electron to the signal
 */
void ChargeSignal::add_electron( const double time ) {

	++_electron_count;
	const double old_mean = _mean_time;
	const double old_variance = _S;

	_mean_time = old_mean + (time - old_mean)/_electron_count;
	_S = old_variance + (time - old_mean)*(time - _mean_time);
}


/** gets the number of electrons
*/
int ChargeSignal::get_electron_count( void ) const {

	return _electron_count;
}


/** gets the unique id of this charge signal
*/
int ChargeSignal::get_charge_id( void ) const {

	return _charge_id;
}


/** gets the pad id of this signal
*/
int ChargeSignal::get_pad_id( void ) const {

	return _pad_id;
}



/** gets the module id of this charge signal
*/
int ChargeSignal::get_module_id( void ) const {

	return _module_id;
}


/** gets the variance in the time
*/
double ChargeSignal::get_time_variance( void ) const {

	if( _electron_count <= 1 ) {

		return _S;
	}
	else {

		return _S/(_electron_count - 1);
	}
}


/** gets the mean time of the cloud
*/
double ChargeSignal::get_mean_time( void ) const {

	return _mean_time;
}


/** returns the number of integers used
*/
int ChargeSignal::getNInt() const {

	return 4;
}


/** returns the number of floats used
*/
int ChargeSignal::getNFloat() const {

	return 0;
}


/** returns the number of doubles used
*/
int ChargeSignal::getNDouble() const {

	return 2;
}


/** returns an int value at a given index
*/
int ChargeSignal::getIntVal( int index ) const {

	switch( index ) {

		case 0: return _electron_count;
		case 1: return _charge_id;
		case 2: return _pad_id;
		case 3: return _module_id;
	}
	return 0;
}


/** returns a float value at a given index
*/
float ChargeSignal::getFloatVal( int index ) const {

	return 0.;
}


/** returns a double value at a given index
*/
double ChargeSignal::getDoubleVal( int index ) const {

	switch( index ) {

		case 0: return _mean_time;
		case 1: return _S;
	}
	return 0.;
}


/** returns whether or not the number of parameters of the signal change during runtime
*/
bool ChargeSignal::isFixedSize() const {

	return true;
}


/** returns the typename of the class
*/
const string ChargeSignal::getTypeName( void ) const {

	return string("ChargeSignal");
}


/** returns the type of data in the generic object
*/
const std::string ChargeSignal::getDataDescription( void ) const {

	return string("i:electron_count;i:charge_id;i:pad_id;i:module_id;d:mean_time;d:time_variance");
}

} // namespace marlintpc
