#include "LCObjectCopier.h"

namespace lcio
{

IMPL::TrackerPulseImpl * LCObjectCopier::copy(const EVENT::TrackerPulse * const inPulse)
{
    IMPL::TrackerPulseImpl * newPulse = new  IMPL::TrackerPulseImpl;

    newPulse->setCellID0(inPulse->getCellID0());

    newPulse->setCellID1(inPulse->getCellID1());

    newPulse->setTime(inPulse->getTime());

    newPulse->setCharge(inPulse->getCharge());

    newPulse->setQuality(inPulse->getQuality());

    newPulse->setCovMatrix(inPulse->getCovMatrix());
    // attention, does only copy the vector
    // this is no clone/ deep copy function
    newPulse->setTrackerData(inPulse->getTrackerData());

    return newPulse;
}

IMPL::TrackImpl * LCObjectCopier::copyTrackWithoutHits(const EVENT::Track * const track)
{
    // Currently (v01-11) there is no copy constructor, so the default cc does exactly what we want:
    // A shallow copy, the contained subtracks and TrackerHis do  not belong to the track anyhow.

    IMPL::TrackImpl * newTrack = new  IMPL::TrackImpl(); //*dynamic_cast<IMPL::TrackImpl const *>(track));
    int typeword = track->getType();
    for (unsigned int i = 0 ; i < 32 ; i++)
        {
            if (typeword & (1 < i))
                {
                    newTrack->setTypeBit(i);
                }
        }

    newTrack->setD0(track->getD0());
    newTrack->setPhi(track->getPhi());
    newTrack->setOmega(track->getOmega());
    newTrack->setZ0(track->getZ0());
    newTrack->setTanLambda(track->getTanLambda());
    newTrack->setCovMatrix(track->getCovMatrix());

    // there is a bug in lcio, setReferencePoint should take a const pointer
    // newTrack->setReferencePoint( track->getReferencePoint() );
    // As it doesn't we have to programm around it
    // newTrack->setReferencePoint( const_cast<float *>(track->getReferencePoint()) );
    // This would work, but it's realy evil, I don't like it
    // This is better:
    const float * constRefPoint = track->getReferencePoint();
    float nonConstRefPoint[3] = {constRefPoint[0],
                                 constRefPoint[1],
                                 constRefPoint[2]
                                };
    newTrack->setReferencePoint(nonConstRefPoint);

    // fg: ---- no longer available in LCIO v02-00 
    // newTrack->setIsReferencePointPCA(track->isReferencePointPCA());

    newTrack->setChi2(track->getChi2());
    newTrack->setNdf(track->getNdf());
    newTrack->setdEdx(track->getdEdx());
    newTrack->setdEdxError(track->getdEdxError());

    const EVENT::TrackVec & trackVec = track->getTracks();
    for (EVENT::TrackVec::const_iterator trackIter = trackVec.begin();
            trackIter < trackVec.end(); trackIter++)
        {
            newTrack->addTrack(*trackIter);
        }

    // Does it make sense to copy this when the hits are not copied?
    // They are set manually, so we just keep them...
    newTrack->setRadiusOfInnermostHit(track->getRadiusOfInnermostHit());
    newTrack->subdetectorHitNumbers() = track->getSubdetectorHitNumbers();

    return newTrack;
}

IMPL::TrackImpl * LCObjectCopier::copy(const EVENT::Track * const track)
{
    // first do a copy without the hits, add them afterwards
    IMPL::TrackImpl * newTrack = copyTrackWithoutHits(track);

    const EVENT::TrackerHitVec & hitVec = track->getTrackerHits();
    for (EVENT::TrackerHitVec::const_iterator hitIter = hitVec.begin();
            hitIter < hitVec.end(); hitIter++)
        {
            newTrack->addHit(*hitIter);
        }
    return newTrack;
}


IMPL::TrackerDataImpl* LCObjectCopier::copy(const EVENT::TrackerData* const inData)
{

    IMPL::TrackerDataImpl* newData = new IMPL::TrackerDataImpl;

    newData->setCellID0(inData->getCellID0());
    newData->setCellID1(inData->getCellID1());
    newData->setTime(inData->getTime());
    newData->setChargeValues(inData->getChargeValues());

    return newData;
}

IMPL::MCParticleImpl* LCObjectCopier::copy(const EVENT::MCParticle* const inParticle)
{

    //ATTENTION THE COPIED MCPARTICLE MISSES MOTHER/CHILD DEPENDENCIES

    double mom[3] =  {inParticle->getMomentum()[0], inParticle->getMomentum()[1], inParticle->getMomentum()[2]};
    double vert[3] = {inParticle->getVertex()[0], inParticle->getVertex()[1], inParticle->getVertex()[2]};
    double ept[3] = {inParticle->getEndpoint()[0], inParticle->getEndpoint()[1], inParticle->getEndpoint()[2]};

    double* momentum = mom;
    double* vertex = vert;
    double* endpoint = ept;

    IMPL::MCParticleImpl* newParticle = new IMPL::MCParticleImpl;

    newParticle->setCharge(inParticle->getCharge());
    newParticle->setMass(inParticle->getMass());
    newParticle->setMomentum(momentum);
    newParticle->setEndpoint(endpoint);
    newParticle->setTime(inParticle->getTime());
    newParticle->setVertex(vertex);
    newParticle->setSimulatorStatus(newParticle->getSimulatorStatus());
    newParticle->setGeneratorStatus(inParticle->getGeneratorStatus());
    newParticle->setPDG(inParticle->getPDG());
    newParticle->setCreatedInSimulation(inParticle->isCreatedInSimulation());
    newParticle->setStopped(inParticle->isStopped());
    newParticle->setHasLeftDetector(inParticle->hasLeftDetector());
    newParticle->setBackscatter(inParticle->isBackscatter());
    newParticle->setVertexIsNotEndpointOfParent(inParticle->vertexIsNotEndpointOfParent());
    newParticle->setDecayedInTracker(inParticle->isDecayedInTracker());
    newParticle->setDecayedInCalorimeter(inParticle->isDecayedInCalorimeter());

    return newParticle;
}


IMPL::SimTrackerHitImpl* LCObjectCopier::copy(const EVENT::SimTrackerHit* const inHit)
{

    IMPL::SimTrackerHitImpl* newHit = new IMPL::SimTrackerHitImpl;

    double position[3] = {inHit->getPosition()[0], inHit->getPosition()[1], inHit->getPosition()[2]};
    double* pos = position;

    float momentum[3] = {inHit->getMomentum()[0], inHit->getMomentum()[1], inHit->getMomentum()[2]};
    float* mom = momentum;

    newHit->setCellID(inHit->getCellID());
    newHit->setPosition(pos);
    newHit->setdEdx(inHit->getdEdx());
    newHit->setTime(inHit->getTime());

    //ATTEENTION THE COPIED MCPARTICLE MISSES MOTHER CHILD DEPENDENCIES

    //newHit->setMCParticle( LCObjectCopier::copy(inHit->getMCParticle()) );
    // Do NOT copy the MCParticel in memory, it is not owned by the SimTrackerHit
    // Just copy the link
    newHit->setMCParticle(inHit->getMCParticle());

    newHit->setMomentum(mom);
    newHit->setPathLength(inHit->getPathLength());

    return newHit;

}


}// namespace lcio
