#ifndef TimePixDistrDelayCorrectionMapReaderProcessor_h
#define TimePixDistrDelayCorrectionMapReaderProcessor_h 1

#include "marlin/DataSourceProcessor.h"
#include <fstream>

namespace marlin{
  
  /** Reads TimePix calibration data: matrix of 256 x 256 values stored in a .txt file\n
   *  Writes data to LCGenericObject Collection named CalibrationData.
   *  When applying to a quad board need to make sure that input file will have 256 x 1024 format
   *  with (1)chip1 (2)chip0 (3)chip2 (4)chip3  calibration maps, one below the other.
   *  
   *
   *  <h4>Input - Prerequisites</h4>
   *  TimePix calibration data files.
   *
   *  <h4>Output</h4>
   *  \li A runheader
   *  \li Collection with LCGenericObject
   *
   * @param  TimeWalkCorrectionMapFileName  input file with TimeWalkCorrectionMap
   * \alert{TimePix file names must be absolute pathes!}
   * @param OutputCollectionName Name of the output collection, defaut  CalibrationData
   * @param DetectorName Detector name for run header, default "TPC"
   * @param Description  Data descripton for run header, default "no description given"
   * @param SubDetectorName Sub-detector name for run header, default "TimePix"
   * @param ChipIDs A list of Chip IDs for which a calibration map is read 
   * @param NRow Number of rows on the chip, default 256
   * @param NColumn Number of columns on the chip, default 256
   * @param SetOutputTransient If not 0 the output collection is set transient
   * @param TimePixChipID Identification name of chip
   *
   * @author R.Ulman , Bonn
   */
  
  class TimePixDistrDelayCorrectionMapReaderProcessor : public DataSourceProcessor {
    
  public:

   TimePixDistrDelayCorrectionMapReaderProcessor() ;

    TimePixDistrDelayCorrectionMapReaderProcessor*  newProcessor() ;


    /** Creates event with LCGenericObject collection from the Calibration data input file
     *  where there's a calibration value for each pixel (for clock distribution correction)
     *  and calls all active processors' processEvent() and processRunHeader Method.
     *
     */
    void readDataSource( int numEvents ) ;
    
    
    void init() ;
    void end() ;

     
  protected:
    
    std::string _outputColName;

    std::string _TimeWalkCorrectionMapFileName;
    std::ifstream _infile;
    std::vector<int> _chipIDs;
    int _runNr;
    int _evtNr;
    int _nRow;
    int _nColumn;
    std::string _detName;
    std::string _dataDescription;
    std::string _subDetName;
    std::string _timePixChipID;
    
    /** give the status which is set to the transient flag of the output collection
     * it is an int instead of a boolean, because the processor parameter cannot be a boolean
     */
    bool _outputIsTransient;
    
  };

 
} // end namespace marlin 

#endif
