#include "AddEmptyPedestalsProcessor.h"
#include <iostream>
#include <string>

//GEAR
#include <gear/TPCParameters.h>
#include <gear/PadRowLayout2D.h>
#include <gear/TPCModule.h>

//tpcconddata
#include "Pedestal.h"

//LCIO
#include <IMPL/LCCollectionVec.h>

//Marlin
#include <marlin/Global.h>

//lccd
#include <lccd.h>
#include <lccd/LCConditionsMgr.hh>
#include <lccd/DataFileHandler.hh>

using namespace lcio;
using namespace marlin;
using namespace std;
using namespace gear;
using namespace tpcconddata;


AddEmptyPedestalsProcessor aAddEmptyPedestalsProcessor;


AddEmptyPedestalsProcessor::AddEmptyPedestalsProcessor() 
  : Processor("AddEmptyPedestalsProcessor") 
  , _isFirstEventInRun(true)
{
  // the processor description
  _description = "AddEmptyPedestalsProcessor ";
  
  registerOutputCollection(   LCIO::LCGENERICOBJECT ,
			      "PedestalCollectionName" ,
			      string("Name of the output collection")+
			      string("with tpcconddata::Pedestal objects, default: TPCPedestals"),
			      _pedestalColName ,
			      string("TPCPedestals")) ;
}

AddEmptyPedestalsProcessor::~AddEmptyPedestalsProcessor(){}


void AddEmptyPedestalsProcessor::init()
{ 
  // you might want to list the used parameters by invoking
  printParameters();
}


void AddEmptyPedestalsProcessor::processRunHeader(LCRunHeader* run)
{
  // do something for each run
  // set the  _isFirstEventInRun flag to true
  _isFirstEventInRun=true;
} 


void AddEmptyPedestalsProcessor::processEvent(LCEvent* evt)
{ 
  // Add the pedestals to the first event.

  if(  _isFirstEventInRun )
  {
    _isFirstEventInRun = false;
    
    //Create a collection of pedestals in first event
    LCCollectionVec* pedestalCol = new LCCollectionVec( LCIO::LCGENERICOBJECT );
	  
    // loop all modules in the TPC and add pedestals for each pad
    const std::vector< TPCModule * > & moduleVec = Global::GEAR->getTPCParameters().getModules();

    for ( std::vector< TPCModule * >::const_iterator moduleIter = moduleVec.begin() ;
	  moduleIter < moduleVec.end(); moduleIter++ )
    {
      TPCModule * module = *moduleIter;
    
      //loop the rows
      for( int row = 0; row < module->getNRows(); row++ )
      {

	//Vector of the pad indices in this row
	std::vector<int> padsInRowVector = module->getPadsInRow( row );
	
	//Now loop all pads in row
	for( vector<int>::iterator padIndexIter=padsInRowVector.begin();
	     padIndexIter < padsInRowVector.end(); 
	     padIndexIter++  )
	{
	  pedestalCol->push_back( new Pedestal(std::make_pair(*padIndexIter, module->getModuleID() )) ) ; 
	} // for padIndexIter
      }// for row
    }// for moduleIter

    //Add pedestal collection with trivial mapping to the event
    evt->addCollection( pedestalCol , _pedestalColName ) ;   

    //Create a lccd::DataFileHandler 
    lccd::LCConditionsMgr::instance()->registerHandler( _pedestalColName , new lccd::DataFileHandler( _pedestalColName ) );

    lccd::LCConditionsMgr::instance()->getHandler( _pedestalColName )->updateEvent(  evt ) ;
  } // if eventNumber==0
}


void AddEmptyPedestalsProcessor::check(LCEvent* evt)
{ 
  // check what is needed here 
}


void AddEmptyPedestalsProcessor::end()
{ 
  // do something at the end of the processing, e.g. cleaning up
}
