#include "LoadDataFileProcessor.h"

//C++
#include <iostream>
#include <cmath>

//LCIO
//#include <IMPL/LCCollectionVec.h>
#include <IO/LCReader.h>

//marlin
#include <marlin/Global.h>

using namespace lcio ;
using namespace marlin ;

namespace marlintpc
{

LoadDataFileProcessor aLoadDataFileProcessor;
    
    LoadDataFileProcessor::LoadDataFileProcessor(): Processor("LoadDataFileProcessor"),
						    _lcReader(0),
						    _run(0)

{
    // register steering parameters: name, description, class-variable, default value
    registerProcessorParameter( "FilePath" , 
				"Path of the file to be read in "  ,
				_filePath,
				std::string("inputFile.slcio") ) ; 
	   
    

    // modify processor description
    _description = "LoadDataFileProcessor: reads in an additional lcio file and adds all available collections "
	"to the event with the same event number";



}


LoadDataFileProcessor::~LoadDataFileProcessor(){}


void LoadDataFileProcessor::init(){

    
    // usually a good idea to
    printParameters() ;
    // create lcio reader 
    _lcReader = LCFactory::getInstance()->createLCReader() ;
   
 
//    _lcReader->registerLCRunListener( ProcessorMgr::instance() ) ; 
//    _lcReader->registerLCEventListener( ProcessorMgr::instance() ) ; 
// 
 
   // open data
   _lcReader->open( _filePath ) ; 
 
 
}



void LoadDataFileProcessor::processRunHeader( EVENT::LCRunHeader* run ){

    run->parameters().setValue( _processorName + "_revision", "$Rev: 2665 $");

    for ( ProcParamMap::iterator i = _map.begin(); i != _map.end(); i++ ) {
	if ( ! i->second->isOptional() || i->second->valueSet() ) {
	    run->parameters().setValue( _processorName + "_" + i->second->name(), 
					i->second->value() );
	}
    }

    _run = run;

}


    
void LoadDataFileProcessor::processEvent( LCEvent * evt ) {
 

    streamlog_out(MESSAGE1)<<"Event "<<evt->getEventNumber()<<std::endl;

   EVENT::LCEvent* newevt=NULL;
   
   streamlog_out(DEBUG4)<<"try to read Run "<<_run->getRunNumber()<<" Event "<<evt->getEventNumber()<<std::endl;

   try{
       newevt = _lcReader->readEvent(_run->getRunNumber(), evt->getEventNumber() );
   }
   catch( std::exception& e){
       
       std::cout << "Warning: " << e.what() << std::endl ;
   }

   
   const std::vector<std::string>* colNames = newevt->getCollectionNames();

   streamlog_out(DEBUG4)<<"outer Event contains "<<colNames->size()<<" collections"<<std::endl;

   for ( std::vector<std::string>::const_iterator i = colNames->begin(); i < colNames->end(); i++ ) {

       streamlog_out(DEBUG4)<<"Collection from extra file: "<< *i <<std::endl;

       evt->addCollection( newevt->takeCollection(*i), *i );
//	   virtual LCCollection * 	takeCollection (const std::string &name) const =0 
//         throw (DataNotAvailableException, std::exception )
// 	   Returns the collection for the given name and transfers the ownership of the collection to the caller.  

   }
    
}

void LoadDataFileProcessor::end(){

    _lcReader->close() ;
    delete _lcReader ;


}


}// namespace 
