#ifndef TPCConditions_h
#define TPCConditions_h 1

#include "iostream"
#include "string"

#include "lcio.h"
#include "UTIL/LCFixedObject.h"


#define TPCConditionsNINTVals 0
#define TPCConditionsNFLOATVals 7
#define TPCConditionsNDOUBLEVals 0


namespace tpcconddata{

class TPCConditions ;

class TPCConditions : public UTIL::LCFixedObject<TPCConditionsNINTVals,
		      TPCConditionsNFLOATVals,TPCConditionsNDOUBLEVals> {
  
public: 
  
    /** Convenient constructor.
     */
    TPCConditions();
    
    /** 'Copy constructor' needed to interpret LCCollection read from file/database.
     */
    TPCConditions(EVENT::LCObject* obj) : UTIL::LCFixedObject<TPCConditionsNINTVals,
							      TPCConditionsNFLOATVals,
							      TPCConditionsNDOUBLEVals>(obj) { } 
    
    /** Important for memory handling*/
    virtual ~TPCConditions();
    
    // the class interface:
    
    /// return the drift velocity (in mm/us)
    float getDriftVelocity() const;

    /// return the diffusion coefficent for the longitudinal diffusion (in sqrt(mm))
    float getLongDiffusionCoef() const;
 
    /// return the defocussion constant for the longitudinal diffusion (in mm)
    float getLongDefocussing() const;
 
    /// return the diffusion coefficent for the transversal diffusion (in sqrt(mm)) 
    float getTransDiffusionCoef() const;
    
    /// return the defocussion constant for the transversal diffusion (in mm)
    float getTransDefocussing() const;
    
    /// return the amplification of the gas amplification device (e.g. GEMs)
    float getAmplification() const;
    
    /// return the conversion factor from ADC counts to primary electrons (ADC * conv.Factor = # primary e)
    float getADCtoPrimaryElectronsFactor() const;
    
    /// set the drift velocity in mm/us
    void setDriftVelocity(float Value);

    /// set the diffusion coefficient for the longitudinal diffusion (in sqrt(mm))
    void setLongDiffusionCoef(float Value);

    /// set the defocussion constant for the longitudinal diffusion (in mm)
    void setLongDefocussing(float Value);
    
    /// set the diffusion coefficient for the transversal diffusion (in sqrt(mm)) 
    void setTransDiffusionCoef(float Value);
    
    /// set the defocussion constant for the transversal diffusion (in mm)
    void setTransDefocussing(float Value) ;
    
    /// set the amplification of the gas amplification device (e.g. GEMs)
    void setAmplification(float Value);
    
    /// set the conversion factor from ADC counts to primary electrons (ADC * conv.Factor = # primary e)
    void setADCtoPrimaryElectronsFactor(float Value);
    
    static std::string getRevision();
    static std::string getDefaultColName();
    
    void print( std::ostream& os = std::cout ) const;
    
    // -------- need to implement abstract methods from LCGenericObject
    const std::string getTypeName() const {
	return "TPCConditions";
    } 
    const std::string getDataDescription() const {
	return std::string("f:Driftvelocity,longDiffusionCoef,longDefocussing,")+
	    std::string("transDiffusionCoef,transDefocussing,Amplification");
    }
}; // class

    std::ostream &operator<<(std::ostream &os, const TPCConditions &tc);
}
#endif
