#ifndef DUMP_SIM_TRACKER_HITS_PROCESSOR_H
#define DUMP_SIM_TRACKER_HITS_PROCESSOR_H 1

//LCIO
#include <lcio.h>

//MARLIN
#include <marlin/Processor.h>

namespace marlintpc
{

    /** Dump the coordinates of all hits in a SimTrackerHits collection.
     *  This is easier to read than the output of dumpevent.
     *
     *  In addition information about the MCParticle which caused the
     *  hit can be printed.
     *
     * @param InputHitsCollection Name of the input SimTrackerHits collection, default: TPCSimHits
     * @param PrintMCParticleInfo Flag whether information about the particle which caused this hit should be printed, default: false
     *
     *  @author Martin Killenberg, University of Bonn
     */
    class DumpSimTrackerHitsProcessor : public marlin::Processor{
			      
    public:
    
	virtual Processor*  newProcessor() { return new DumpSimTrackerHitsProcessor ; }
    
	DumpSimTrackerHitsProcessor();

	virtual void processEvent( LCEvent * evt ) ; 
    
    protected:

     std::string _inputColName ;    ///< the name of the input collection
     bool _printMCParticleInfo; ///< flag wheter to print information about the MCPArticles

    };
}

#endif // DUMP_SIM_TRACKER_HITS_PROCESSOR_H
